package	jp.co.sds_corp.ratemixer.fx.source;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.FileReader;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Map;

import jp.co.sds_corp.fxutil.CcyPair;
import jp.co.sds_corp.fxutil.MasterCache;
import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* ダミーのレートを送信します。
* このクラスはテスト用です。
<PRE>
&lt;Source name="Dummy" class="jp.co.sds_corp.ratemixer.fx.source.DummyFileSource"&gt;
	&lt;file&gt;data/test.dat&lt;/file&gt;
	&lt;interval type="real"/&gt;
	&lt;interval type="sec"&gt;3&lt;/interval&gt;
&lt;/Source&gt;
</PRE>
*/
public class DummyFileSource extends FxSource
{
	private static final long serialVersionUID = 1L;
	String	file;
	transient ServiceThread thr;
	MasterCache master;

	/**
	* 指定された初期情報でDummySourceを初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		file = config.get( "file", null );
		master = MasterCache.getInstance();
	}

	/**
	* このサービスを開始します。
	* 裏Threadを起動し、指定された秒数間隔でダミーのレートを送信します。
	*/
	public void start() {
		thr = new ServiceThread();
		thr.setName( getName() );
		thr.start();
		setInterrupt( false );
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		setInterrupt( true );
		thr.interrupt();
		try {
			thr.join();
		} catch ( Exception ex ) {
		}
		CmnLog.debug( this, "stopped." );
	}

	/**
	* ファイルの行を、{@link FxRate}に変換します。
	* @param	buf		行の内容
	* @return	変換されたレート
	*/
	protected FxRate parseLine( String buf ) throws Exception {
		String[] line = CmnString.getStringToken( buf, "\t" );
		String term = line[2];
		BigDecimal bid = new BigDecimal( line[3] );
		BigDecimal ask = new BigDecimal( line[4] );
		String upd = line[5];

/*
		Ccy ccy1 = new Ccy( line[0] );
		Ccy ccy2 = new Ccy( line[1] );
		CcyPair pair = new CcyPair( ccy1, ccy2 );
		pair.setPoint( 4 );
*/
		CcyPair pair = master.getCcyPair( line[0], line[1] );
		if ( pair == null ) {
			if ( isDebug() ) {
				CmnLog.debug( this, "CcyPair not found."
									+ line[0] + "/" + line[1] );
			}
			return null;
		}
		FxRate rate = new FxRate( pair, term );
		rate.setBid( bid );
		rate.setAsk( ask );
		rate.setRecvTime( new CmnDate( upd, "yyyy-MM-dd HH:mm:ss" ));
		return rate;
	}

	private class ServiceThread extends Thread
	{
		public ServiceThread() {
		}

		public void run() {
			int interval = 1000;
			Map<String,FxRate> mapRate = getRateMap();

			BufferedReader br = null;
			try {
				br = new BufferedReader( new FileReader( file ));
				for ( ;; ) {
					if ( DummyFileSource.this.isInterrupted()) {
						break;
					}
					String line = br.readLine();
					if ( line == null ) {
						throw( new EOFException( "EOF" ));
					}

					FxRate newRate = parseLine( line );
					if ( newRate == null ) {
						continue;
					}
					String key = newRate.getKey().toString();

					fireReceive( newRate );
					mapRate.put( key, newRate );

					try {
						Thread.sleep( interval );
					} catch ( Exception ex ) {
					}
				}
			} catch ( Exception ex ) {
				CmnLog.error( this, "error", ex );
			} finally {
				if ( br != null ) {
					try {
						br.close();
					} catch ( IOException ex ) {
						CmnLog.error( this, "close error", ex );
					}
				}
			}
		}
	}

}
