package	jp.co.sds_corp.ratemixer.fx.source;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;

import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* HTTPのレートソースからレートを取得するオブジェクトの基底クラスです。
* このクラスでは、定期的にHTTPリクエストを送信するThreadを起動します。
*/
public abstract class HttpSource extends FxSource
{
	private static final long serialVersionUID = 1L;
	boolean		bChg;
	URL			url;
	String		userAgent;
	int			interval;
	int			timeout;
	int			status = 0;
	transient ServiceThread thr;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		url = new URL( config.get( "url" ));
		userAgent = config.get( "userAgent", null );
		timeout = config.getInteger( "timeout", 0 );
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
		thr = new ServiceThread();
		thr.setName( getName() );
		thr.start();
		setInterrupt( false );
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		setInterrupt( true );
		if ( thr != null ) {
			thr.interrupt();
			try {
				thr.join();
			} catch ( Exception ex ) {
			}
		}
		CmnLog.debug( this, "stopped." );
	}

	private class ServiceThread extends Thread
	{
		public ServiceThread() {
		}

		public void run() {
			int		nSleep = 5000;
			int		nCnt = 0;
			boolean	bValid = isValid();
			for ( ;; ) {
				if ( HttpSource.this.isInterrupted()) {
					break;
				}
				if ( !isValid() ) {
					if ( bValid ) {
						CmnLog.info( this, "isValid() change to false." );
					}
					bValid = false;
					try {
						Thread.sleep( 60000 );
					} catch ( Exception ex ) {
					}
					continue;
				}
				if ( !bValid ) {
					CmnLog.info( this, "isValid() change to true." );
				}
				bValid = true;
				bChg = false;
				getRates();

				if ( bChg ) {
					nCnt = 0;
					nSleep = 1000;
//CmnLog.debug( this, "sleep=" + nSleep + " ----" );
				} else {
					nCnt++;
					if ( nSleep < 5000 ) {
						nSleep = 5000;
//CmnLog.debug( this, "sleep=" + nSleep + " ----" );
					}
					if ( nCnt > 10 ) {
						nCnt = 0;
						if ( nSleep < 60000 ) {
							nSleep += 5000;
CmnLog.debug( this, "sleep=" + nSleep + " ----" );
						}
					}
				}
				try {
					Thread.sleep( nSleep );
				} catch ( Exception ex ) {
				}
			}
		}
	}

	/**
	* HTTPリクエストを送信します。
	* @param	os		出力先
	*/
	public abstract void sendRequest( OutputStream os ) throws Exception;
	/**
	* HTTPレスポンスを解析します。
	* @param	res		HTTPレスポンスの内容
	*/
	public abstract void analyze( String res ) throws Exception;

	private void getRates() {
		Receiver rcv = new Receiver( url );
		rcv.setName( getName() + ":recv" );
		rcv.start();
		try {
			rcv.join( timeout );
		} catch ( Exception ex ) {
		}
		if ( rcv.isAlive() ) {
			setStatus( -1, "interrupt", new InterruptedException());
			rcv.interrupt();
		}
		String res = rcv.getResult();
		if ( res == null ) {
			// setStatus( -1, "receive error", new NullPointerException());
		} else {
			try {
				analyze( res );
			} catch ( Exception ex ) {
				setStatus( -1, "analyze error", ex );
			}
		}
	}

	private synchronized void setStatus( int status, String msg, Exception ex ) {
		if ( status != 0 ) {
			if ( this.status == 0 ) {
				CmnLog.error( this, msg, ex );
				StatusChangeEvent ev = new StatusChangeEvent( this,
					StatusChangeEvent.STAT_RUN, StatusChangeEvent.STAT_ERR,
					msg );
				try {
					fireStatusChange( ev );
				} catch ( Exception ee ) {
				}
			} else {
				CmnLog.warn( this, msg, ex );
			}
		} else {
			if ( this.status != 0 ) {
				CmnLog.info( this, "wake up" );
				StatusChangeEvent ev = new StatusChangeEvent( this,
					StatusChangeEvent.STAT_ERR, StatusChangeEvent.STAT_RUN,
					"recovered." );
				try {
					fireStatusChange( ev );
				} catch ( Exception ee ) {
				}
			}
		}
		this.status = status;
	}

	private class Receiver extends Thread {
		URL			url;
		String		res;

		public Receiver( URL url ) {
			this.url = url;
		}

		String getResult() {
			return res;
		}

		public void run() {
			InputStreamReader reader = null;
			InputStream is = null;
			try {
				res = null;
				URLConnection conn = url.openConnection();
				conn.setDoOutput(true);
				conn.setDoInput(true);
				conn.setUseCaches(false);
				conn.setAllowUserInteraction(false);
				conn.setRequestProperty("Content-type", "text/plain");
				if ( userAgent != null ) {
					conn.setRequestProperty( "User-Agent", userAgent );
				}
				//java.io.OutputStream os = SecuritySetup.openPlainOutputStream(conn);
				OutputStream os = conn.getOutputStream();
				sendRequest( os );

				is = conn.getInputStream();
				reader = new InputStreamReader(is);
				StringBuffer rb = new StringBuffer();
				int b = -1;
				int bufferLength = 1024;
				char buffer[] = new char[bufferLength];
				do {
					b = reader.read( buffer, 0, bufferLength );
					if(b >= 0)
						rb.append( buffer, 0, b );
				} while ( b >= 0 );
				res = rb.toString();
				setStatus( 0, "", null );
			} catch( Exception ex ) {
				setStatus( -1, "receive error", ex );
			} finally {
				if ( reader != null ) {
					try {
						reader.close();
					} catch (IOException ex) {
						CmnLog.error( this, "close error", ex );
					}
				}
				if ( is != null ) {
					try {
						is.close();
					} catch (IOException ex) {
						CmnLog.error( this, "close error", ex );
					}
				}
			}
		}

	}

}
